/*

	HEAVILY MODIFIED BY WIDGETSCHMIEDE ARTISTS =)
	******************************************



File: Scroller.js

Abstract: JavaScript support code for a DHTML scrollbar.  It should not be 
	necessary to understand this code if the instructions in Scroller.html are
	followed.  The only things of interest for customization should be the 
	LINE_SCROLL_DIST (number of pixels each up/down arrow stroke scrolls)
	and PAGE_SKIP_PAUSE (time interval when holding the mouse down in the 
	scroll track, or holding the page/arrow keys) constants.
	
	To change the content currently controlled by the Scroller, pass the new 
	content div to calculateAndShowThumb (see ScrollerMain.js)

Version: 1.0

� Copyright 2005 Apple Computer, Inc. All rights reserved.

IMPORTANT:  This Apple software is supplied to 
you by Apple Computer, Inc. ("Apple") in 
consideration of your agreement to the following 
terms, and your use, installation, modification 
or redistribution of this Apple software 
constitutes acceptance of these terms.  If you do 
not agree with these terms, please do not use, 
install, modify or redistribute this Apple 
software.

In consideration of your agreement to abide by 
the following terms, and subject to these terms, 
Apple grants you a personal, non-exclusive 
license, under Apple's copyrights in this 
original Apple software (the "Apple Software"), 
to use, reproduce, modify and redistribute the 
Apple Software, with or without modifications, in 
source and/or binary forms; provided that if you 
redistribute the Apple Software in its entirety 
and without modifications, you must retain this 
notice and the following text and disclaimers in 
all such redistributions of the Apple Software. 
Neither the name, trademarks, service marks or 
logos of Apple Computer, Inc. may be used to 
endorse or promote products derived from the 
Apple Software without specific prior written 
permission from Apple.  Except as expressly 
stated in this notice, no other rights or 
licenses, express or implied, are granted by 
Apple herein, including but not limited to any 
patent rights that may be infringed by your 
derivative works or by other works in which the 
Apple Software may be incorporated.

The Apple Software is provided by Apple on an "AS 
IS" basis.  APPLE MAKES NO WARRANTIES, EXPRESS OR 
IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED 
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY 
AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING 
THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE 
OR IN COMBINATION WITH YOUR PRODUCTS.

IN NO EVENT SHALL APPLE BE LIABLE FOR ANY 
SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF 
THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER 
UNDER THEORY OF CONTRACT, TORT (INCLUDING 
NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN 
IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF 
SUCH DAMAGE.

*/


/*
 ********************************
 * Objects used by the Scroller	*
 ********************************
 */
var scrollerStack = new Object();
var lastActiveScrollerID = '';
var scrollTimer = {
	mouseAction:null,
	keyAction:null,
	timer:null
};

/*
	var currentContent;			// currently-visible div; allows Scroller to control arbitrary content
	var scrollBar;				// Parent scrollbar div, contains the track and thumb
	var scrollThumb;			// Scroller's thumb control
	var scrollTrack;			// Scroller's base/track
	var scrollTimer;			// timer for track/key scrolling
*/
/*
 ************************************
 *  Dimensions used by the Scroller	*
 ************************************
 */
/*
	var currentContentHeight;					// height of currently-visible content div
	var currentContentTop;						// top " " "
	var viewHeight;								// height of the parent (overflow:hidden) view
	var scrollBarHeight;						// " " " scrollbar	
	var thumbHeight;							// " " " thumb control
	var numberOfScrollablePixels;				// for calculating thumb size/position and content position ("page number")

	var trackMouseY					= 0;		// mouse location in the scroll track; used for click-scrolling
	var thumbStartY					= -1;		// exact point we started scrolling with the thumb
	var thumbStartTop				= -1;		// thumb's "top" value when we started scrolling
	
	var trackingMouse				= false;	// is the mouse down in the scroll track
	var trackingKeys				= false;	// is arrow/page up/down being held
*/

/*
 ************************************************************
 * Style constants hardcoded to match respective CSS values	*
 ************************************************************
 */
	var SCROLLBAR_TOP			= -1;	// starting (topmost) position for scroll thumb
	var MIN_SCROLL_THUMB_HEIGHT	= 27;	// minimum size for acceptable appearance of the scroll thumb

/*
 ************************************************
 * Key/track scrolling constants				*
 * Customize these according to your content	*
 ************************************************
 */
	var LINE_SCROLL_DIST		= 40;	// line up/down travel; for arrow key scrolling
	var PAGE_SKIP_PAUSE			= 150;	// time (ms) between key/track scroll actions


// set up the scrollbar with elements for the scrollbar, track, thumb
// set scrollable content in calculateAndShowThumb
function scrollerInit (scrollerID, barDiv, trackDiv, thumbDiv) {
	var scrollerControl = { currentContent:null, scrollBar:barDiv, scrollThumb:thumbDiv, scrollTrack:trackDiv, scrollTimer:scrollTimer,
							currentContentHeight:null, currentContentTop:null, viewHeight:null, scrollBarHeight:null, thumbHeight:null, numberOfScrollablePixels:null,
							trackMouseY:0, thumbStartY:-1, thumbStartTop:-1, trackingMouse:false, trackingKeys:false }
	scrollerStack[scrollerID] = scrollerControl;
	
	// keydown listener is in event bubble mode, so child elements of the
	// document can still listen to keys if they need to.
	document.addEventListener("keydown", keyPressed, false);
	// thumb/track listeners are capture-style, to prevent child elements from
	// also reacting to the mouse activity on the scroll track or thumb.
	scrollerControl.scrollTrack.addEventListener("mousedown", mouseDownTrack, true);
	scrollerControl.scrollThumb.addEventListener("mousedown", mouseDownThumb, true);
}

// Calculate the height of the views and make the thumb proportional.
// If a single scroller is being shared across multiple divs (as in this sample)
// this function must be called whenever the divs swap, to recalibrate the scrollbar
// Also must be called when changing the size of the parent div
function calculateAndShowThumb(scrollerID, contentDiv) {
	if (contentDiv != null) {
		scrollerStack[scrollerID].currentContent = contentDiv;
	} else if (scrollerStack[scrollerID].currentContent == null) {
		hideScrollBar(scrollerID);
		return;
	}
	
	//alert("contentDiv: "+contentDiv + " >> id: "+contentDiv.id);
	
	scrollerStack[scrollerID].currentContent.style.display = "block";
	var currentContentStyle = document.defaultView.getComputedStyle(scrollerStack[scrollerID].currentContent,"");
	
	// DEBUG!
	if (null == currentContentStyle) 
	{
		//alert("currentContentStyle was null in Scroller.js -- Shouldn't happen. Resuming.");
		return;
	}
	// END DEBUG
	
	//alert("calculateandshowthumb d >> "+currentContentStyle);
	scrollerStack[scrollerID].currentContentTop = parseInt(currentContentStyle.getPropertyValue("top"));
	if (isNaN(scrollerStack[scrollerID].currentContentTop)) {
		scrollerStack[scrollerID].currentContentTop = 0;
	}

	// Get the proportion of visible content in parent div to total content in child div
	// We'll use this to calculate the size of (and need for) the scroll thumb
	scrollerStack[scrollerID].currentContentHeight = parseInt(currentContentStyle.getPropertyValue("height"));
	scrollerStack[scrollerID].viewHeight = parseFloat (document.defaultView.getComputedStyle(scrollerStack[scrollerID].currentContent.parentNode, "").getPropertyValue("height"));
	scrollerStack[scrollerID].scrollBarHeight = parseInt(document.defaultView.getComputedStyle(scrollerStack[scrollerID].scrollBar, "").getPropertyValue("height"));
	DEBUG("cast: viewHeight=" + scrollerStack[scrollerID].viewHeight + " currentContentHeight=" + scrollerStack[scrollerID].currentContentHeight + " top=" + scrollerStack[scrollerID].currentContentTop + " scrollBarHeight=" + scrollerStack[scrollerID].scrollBarHeight);
		
	var percent = getProportion (scrollerStack[scrollerID].viewHeight, scrollerStack[scrollerID].currentContentHeight);
	DEBUG("cast: percent=" + percent);
	
	// hide the scrollbar if all the content is showing.  Determined by the calculated scrollbar height and position.
	if (percent == 0) {
		scrollerStack[scrollerID].currentContent.style.top = "0px";
		hideScrollBar(scrollerID);
	} else {
		// Position the thumb according to where the content is currently scrolled.
		// This is necessary for sharing the same scrollbar between multiple content
		// panes that will likely be at different scroll positions.
		scrollerStack[scrollerID].thumbHeight = Math.max(Math.round(scrollerStack[scrollerID].scrollBarHeight * percent), MIN_SCROLL_THUMB_HEIGHT);
		
		scrollerStack[scrollerID].numberOfScrollablePixels = scrollerStack[scrollerID].scrollBarHeight - scrollerStack[scrollerID].thumbHeight - SCROLLBAR_TOP;
		
		thumbTop = thumbPositionForPagePosition(scrollerID, scrollerStack[scrollerID].currentContentTop);
		
		scrollerStack[scrollerID].scrollThumb.style.height = scrollerStack[scrollerID].thumbHeight + "px";
		scrollerStack[scrollerID].scrollThumb.style.top = thumbTop;
		
		DEBUG("cast: new scrollerStack[scrollerID].thumbHeight=" + scrollerStack[scrollerID].scrollThumb.style.height + " thumbTop=" + thumbTop);
		
		// This is a safeguard so the content matches the new thumb position.  Necessary for live-resizing to work.
		scrollContent(scrollerID, thumbTop);
		
		showScrollBar(scrollerID);
	}
}

function showScrollBar(scrollerID) {
	scrollerStack[scrollerID].scrollTrack.style.display = "block";
	scrollerStack[scrollerID].scrollThumb.style.display = "block";
}

function hideScrollBar(scrollerID) {
	scrollerStack[scrollerID].scrollTrack.style.display = "none";
	scrollerStack[scrollerID].scrollThumb.style.display = "none";
}

/*
 ****************************
 *	Key Scrolling Functions	*
 ****************************
 */
 
function keyPressed (event) {
	var scrollerID = event.target.hasAttribute('id') ? event.target.getAttribute('id') : lastActiveScrollerID;
	lastActiveScrollerID = scrollerID;
	if(!scrollerID)
		return;
	
	var theAction = null;
	switch (event.charCode) {
		case 63232: // up arrow
			theAction = lineUp;
			break;
		case 63233: // down arrow
			theAction = lineDown;
			break;
		// For home and end, scroll by the known size of the content;
		// this will likely put us out of bounds, but scrollBy will 
		// catch and correct that
		case 63273: // home
			scrollBy(scrollerID, scrollerStack[scrollerID].currentContentHeight);
			theAction = null;
			break;
		case 63275:	// end
			scrollBy(scrollerID, -scrollerStack[scrollerID].currentContentHeight);
			theAction = null;
			break;
		case 63276: // pageup
			theAction = pageUp;
			break;
		case 63277: // pagedn
			theAction = pageDown;
			break;
		default:
			theAction = null;
			break;
	}	
	
	if (theAction == null) {
		if(scrollerStack[scrollerID]) {
			scrollerStack[scrollerID].trackingKeys = false;
		}
	}
	else {
		// Start listening for the key release
		document.addEventListener("keyup", keyReleased, false);
		event.stopPropagation();
		event.preventDefault();

		scrollerStack[scrollerID].trackingKeys = true;
		theAction(scrollerID);
		// Start a timer with the new key action;
		// any new key action should replace/override the old
		setTimerKeyAction(scrollerID, theAction);
	}
	return true;
}

function keyReleased (event) {
	var scrollerID = event.target.hasAttribute('id') ? event.target.getAttribute('id') : lastActiveScrollerID;
	lastActiveScrollerID = scrollerID;
	scrollerStack[scrollerID].trackingKeys = false;
	scrollerStack[scrollerID].scrollTimer.keyAction = null;

	document.removeEventListener("keyup", keyReleased, false);
	event.stopPropagation();
	event.preventDefault();
}


/*
 ********************************
 *	Thumb Scrolling Functions	*
 ********************************
 */

// mousedown is presumably the start of a thumb drag (scroll) action.
function mouseDownThumb (event) {
	var scrollerID = event.target.parentNode.parentNode.hasAttribute('id') ? event.target.parentNode.parentNode.getAttribute('id') : lastActiveScrollerID;
	lastActiveScrollerID = scrollerID;
	
	// These listeners are only useful while there is mouse activity on the thumb.
	// Add them on mousedown; remove on mouseup.
	document.addEventListener("mousemove", mouseMoveScrollThumb, true);
	document.addEventListener("mouseup", mouseUpScrollThumb, true);
	
	scrollerStack[scrollerID].thumbStartY = event.y;
	scrollerStack[scrollerID].thumbStartTop = parseInt(document.defaultView.getComputedStyle(scrollerStack[scrollerID].scrollThumb,"").getPropertyValue("top"));
	if (isNaN(scrollerStack[scrollerID].thumbStartTop)) {
		scrollerStack[scrollerID].thumbStartTop = -1;
	}
	DEBUG("mdt: scrollerStack[scrollerID].thumbHeight:" + scrollerStack[scrollerID].thumbHeight + " scrollerStack[scrollerID].thumbStartY=" + scrollerStack[scrollerID].thumbStartY + " scrollerStack[scrollerID].thumbStartTop=" + scrollerStack[scrollerID].thumbStartTop);
}

// At this point the scrollThumb is being dragged;  We know this because the 
// mousemove listener is only installed after a mousedown
function mouseMoveScrollThumb (event) {
	var scrollerID = lastActiveScrollerID;
	var deltaY = event.y - scrollerStack[scrollerID].thumbStartY;
	
	var newPosition = scrollerStack[scrollerID].thumbStartTop + deltaY;
	DEBUG("mmst: event.y=" + event.y + " scrollerStack[scrollerID].thumbStartY=" + scrollerStack[scrollerID].thumbStartY + " thumbStart=" + scrollerStack[scrollerID].thumbStartTop);
	scrollContent(scrollerID, newPosition);
}

function mouseUpScrollThumb (event) {
	var scrollerID = lastActiveScrollerID;
	
	// Remove listeners; they'll be re-added on the next mouseDown
	document.removeEventListener("mousemove", mouseMoveScrollThumb, true);
	document.removeEventListener("mouseup", mouseUpScrollThumb, true);
	
	// reset the starting position
	scrollerStack[scrollerID].thumbStartY = -1;
}

/*
 ********************************
 *	Track Scrolling Functions	*
 ********************************
 */

// mousedown should reset the timer no matter what; even if already key-scrolling
function mouseDownTrack (event) {
	var scrollerID = event.target.parentNode.parentNode.hasAttribute('id') ? event.target.parentNode.parentNode.getAttribute('id') : lastActiveScrollerID;
	lastActiveScrollerID = scrollerID;
	scrollerStack[scrollerID].trackingMouse = true;
	updateTrackMouseY(event);
	
	scrollerStack[scrollerID].scrollTrack.addEventListener("mousemove", mouseMoveTrack, true);
	scrollerStack[scrollerID].scrollTrack.addEventListener("mouseover", mouseOverTrack, true);
	scrollerStack[scrollerID].scrollTrack.addEventListener("mouseout", mouseOutTrack, true);
	document.addEventListener("mouseup", mouseUpTrack, true);
	
	// Determine if the click was above or below the thumb;
	// Set the repeat timer to the appropriate page action
	var thumbTop = document.defaultView.getComputedStyle(scrollerStack[scrollerID].scrollThumb,"").getPropertyValue("top");
	var debugStr = ("mdt: mouseY=" + scrollerStack[scrollerID].trackMouseY + " scrollThumbY=" + thumbTop + "; ");
		
	if (scrollerStack[scrollerID].trackMouseY > parseInt(thumbTop)) {
			debugStr += "click BELOW thumb";
		pageDown(scrollerID);
		setTimerMouseAction(scrollerID, pageDown);
	} else {
			debugStr += "click ABOVE thumb";
		pageUp(scrollerID);
		setTimerMouseAction(scrollerID, pageUp);
	}
	
	debugStr += (": newPosition=" + scrollerStack[scrollerID].trackMouseY);
	DEBUG(debugStr);
	
	event.stopPropagation();
	event.preventDefault();
}

function mouseMoveTrack(event) {
	var scrollerID = lastActiveScrollerID;
	
	// If the mouse moved while being held down, update the location
	// so track-scrolling stops in the right place
	updateTrackMouseY(event);
	event.stopPropagation();
	event.preventDefault();

}

function mouseOutTrack(event) {
	var scrollerID = lastActiveScrollerID;
	
	// turn track-scrolling off if the mouse moves out while pressed;
	// the timer keeps firing until the mouse is released
	scrollerStack[scrollerID].trackingMouse = false;
	event.stopPropagation();
	event.preventDefault();
}

function mouseOverTrack(event) {
	var scrollerID = lastActiveScrollerID;
	
	// This will resume track-scrolling: timer is still firing, but
	// pageUp/pageDown are waiting for the mouse to return to the track
	updateTrackMouseY(event);
	scrollerStack[scrollerID].trackingMouse = true;
	event.stopPropagation();
	event.preventDefault();
}

function mouseUpTrack(event) {
	var scrollerID = lastActiveScrollerID;
	
	// scrollTimer will stop track-scrolling when we set these two flags
	scrollerStack[scrollerID].trackingMouse = false;
	scrollerStack[scrollerID].scrollTimer.mouseAction = null;

	// After mouseup, these events are just noise. Remove them; they'll be re-added on the next mouseDown
	scrollerStack[scrollerID].scrollTrack.removeEventListener("mousemove", mouseMoveTrack, true);
	scrollerStack[scrollerID].scrollTrack.removeEventListener("mouseover", mouseOverTrack, true);
	scrollerStack[scrollerID].scrollTrack.removeEventListener("mouseout", mouseOutTrack, true);
	document.removeEventListener("mouseup", mouseUpTrack, true);

	event.stopPropagation();
	event.preventDefault();
}

// correct the coordinates for the sourceEvent so they properly match the source component
// **YOU MAY NEED TO UPDATE THIS FUNCTION** depending on how deeply the scrollbar div is nested
function updateTrackMouseY (event) {
	var scrollerID = lastActiveScrollerID;
	
	DEBUG("utmY: source=" + event.toElement.id + " rawY=" + event.y + " offsetY=" + event.offsetY + " layerY=" + event.layerY + " offsetTop=" + event.toElement.offsetTop);
	scrollerStack[scrollerID].trackMouseY = event.offsetY + event.toElement.offsetTop;
}




/*
 ********************
 * Scroll functions	* 
 ********************
 */ 

// Convenience "line" up/down functions; change LINE_SCROLL_DIST
// to increase/decrease interval
function lineDown (scrollerID) {
	if (scrollerStack[scrollerID].trackingKeys==true) {
		scrollBy(scrollerID, -LINE_SCROLL_DIST);
	}
}

function lineUp (scrollerID) {
	if (scrollerStack[scrollerID].trackingKeys==true) {
		scrollBy(scrollerID, LINE_SCROLL_DIST);
	}
}	

// Reposition the content one page (scrollerStack[scrollerID].viewHeight) upwards; the content's top 
// becomes increasingly NEGATIVE (moves upwards) as we scroll down.	
function pageDown(scrollerID) {
	var thumbStyle = document.defaultView.getComputedStyle(scrollerStack[scrollerID].scrollThumb,"");
	var currentThumbBottom = parseInt(thumbStyle.getPropertyValue("top")) + parseInt(thumbStyle.getPropertyValue("height"));
	// Check for key/track scrolling; prevent out-of-bounds values
	if ((scrollerStack[scrollerID].trackingKeys==true) || (scrollerStack[scrollerID].trackingMouse==true && scrollerStack[scrollerID].trackMouseY>currentThumbBottom)) {
		scrollBy(scrollerID, -scrollerStack[scrollerID].viewHeight);
	}
}

// very similar to pageDown, with some values negated to move the content in a different direction.
function pageUp(scrollerID) {
	if (scrollerStack[scrollerID].trackingKeys==true || (scrollerStack[scrollerID].trackingMouse==true && scrollerStack[scrollerID].trackMouseY<parseInt(scrollerStack[scrollerID].scrollThumb.style.top))) {
		scrollBy(scrollerID, scrollerStack[scrollerID].viewHeight);
	}
}

// scrollBy responds to a pixel delta for the content and adjusts the thumb;
// scrollContent responds to a thumb position and adjusts the content
function scrollBy (scrollerID, delta) {
	if (delta == 0) return;
	
	var newY  = scrollerStack[scrollerID].currentContentTop + delta;
	
	if (delta < 0) { // scroll down
		// Don't scroll lower than the last page
		var lastPageY = -(scrollerStack[scrollerID].currentContentHeight - scrollerStack[scrollerID].viewHeight);
		scrollerStack[scrollerID].currentContentTop = Math.max(lastPageY, newY);
	} else { // scroll up
		// Don't scroll above the top
		scrollerStack[scrollerID].currentContentTop = Math.min(0, newY)
	}
	
	scrollerStack[scrollerID].currentContent.style.top = scrollerStack[scrollerID].currentContentTop + "px";
	if('myScrollBar' == scrollerID) {
		updateMagazineViewToScrollPosition();
	}

	// reposition the scroll thumb based on the new page position
	scrollerStack[scrollerID].scrollThumb.style.top = thumbPositionForPagePosition(scrollerID, scrollerStack[scrollerID].currentContentTop) + "px";
	var thumbBottom = parseInt(scrollerStack[scrollerID].scrollThumb.style.top) + parseInt(scrollerStack[scrollerID].scrollThumb.style.height);
	DEBUG("scrollBy: thumbTop=" + scrollerStack[scrollerID].scrollThumb.style.top + " thumbBottom=" + thumbBottom);
}

// Scroll the content based on the current scroller thumb position
function scrollContent(scrollerID, newThumbPosition) {
	// Correct if we're going to clip above the top or below the bottom
	if (newThumbPosition < SCROLLBAR_TOP) {
		DEBUG("sc: thumb too high (" + newThumbPosition + ")");
		newThumbPosition = SCROLLBAR_TOP;
	} else if ((newThumbPosition + scrollerStack[scrollerID].thumbHeight) > scrollerStack[scrollerID].scrollBarHeight) {
		DEBUG("sc: thumb too low (" + newThumbPosition + ")");
		newThumbPosition = scrollerStack[scrollerID].scrollBarHeight - scrollerStack[scrollerID].thumbHeight;
	}
		
	scrollerStack[scrollerID].scrollThumb.style.top = newThumbPosition + "px";
	
	scrollerStack[scrollerID].currentContentTop = pagePositionForThumbPosition(scrollerID, newThumbPosition);
	DEBUG("sc: thumbTop=" + scrollerStack[scrollerID].scrollThumb.style.top + " scrollerStack[scrollerID].currentContentTop is " + scrollerStack[scrollerID].currentContentTop);
	scrollerStack[scrollerID].currentContent.style.top = scrollerStack[scrollerID].currentContentTop + "px";
	if('myScrollBar' == scrollerID) {
		updateMagazineViewToScrollPosition();
	}
}

/*
 ********************************
 *	Utility / Math functions	*
 ********************************
 */
 
function getProportion (viewHeight, documentheight) {
	if (documentheight <= viewHeight)
		return 0;
	else
		return viewHeight/documentheight;
}

// Given the position of the thumb, tell us what the content top should be.
// This is the key value that allows us to thumb-scroll.
function pagePositionForThumbPosition (scrollerID, thumbPosition) {
	DEBUG("ppftp: tp=" + thumbPosition + " st=" + SCROLLBAR_TOP + " cch=" + scrollerStack[scrollerID].currentContentHeight + " vh=" + scrollerStack[scrollerID].viewHeight + " nsp=" + scrollerStack[scrollerID].numberOfScrollablePixels);
	return -(thumbPosition - SCROLLBAR_TOP) * ((scrollerStack[scrollerID].currentContentHeight - scrollerStack[scrollerID].viewHeight) / scrollerStack[scrollerID].numberOfScrollablePixels);
}

// Given the position of the page, tell us where the thumb should be.
// This is the key value that allows us to track-scroll.
function thumbPositionForPagePosition (scrollerID, pagePosition) {
	DEBUG("tpfpp: pp=" + pagePosition + " cch=" + scrollerStack[scrollerID].currentContentHeight + " vh=" + scrollerStack[scrollerID].viewHeight + " nsp=" + scrollerStack[scrollerID].numberOfScrollablePixels + " st=" + SCROLLBAR_TOP);
	var newThumbY = -(pagePosition / ((scrollerStack[scrollerID].currentContentHeight - scrollerStack[scrollerID].viewHeight) / scrollerStack[scrollerID].numberOfScrollablePixels)) + SCROLLBAR_TOP;
	return Math.max(newThumbY, -1);
}

/*
 ****************************************
 *	Scroll timer for mouse/key tracking	*
 ****************************************
 */
 

function setTimerMouseAction (scrollerID, newAction) {
	scrollerStack[scrollerID].scrollTimer.mouseAction = newAction;
	startTimerWithDelay(scrollerID);
}

function setTimerKeyAction (scrollerID, newAction) {
	scrollerStack[scrollerID].scrollTimer.keyAction = newAction;
	// if a mouseAction is in place, don't reset.
	// A new timer should start either to override an old keyAction,
	// or if there isn't one at all.
	if (scrollerStack[scrollerID].scrollTimer.mouseAction == null) {
		startTimerWithDelay(scrollerID);
	}
}

function startTimerWithDelay (scrollerID) {
	clearTimer(scrollerID);
	window.setTimeout("startTimer('" + scrollerID + "');", PAGE_SKIP_PAUSE*2);
}

function startTimer (scrollerID) {
	clearTimer(scrollerID);
	if (scrollerStack[scrollerID].scrollTimer.mouseAction == null && scrollerStack[scrollerID].scrollTimer.keyAction == null) {
		return;
	}
	scrollerStack[scrollerID].scrollTimer.timer = window.setInterval("doTimerAction('" + scrollerID + "');", PAGE_SKIP_PAUSE);
}

function clearTimer (scrollerID) {
	window.clearInterval(scrollerStack[scrollerID].scrollTimer.timer);
	scrollerStack[scrollerID].scrollTimer.timer = null;
}

function doTimerAction (scrollerID) {
	// mouseAction always trumps keyAction
	if (scrollerStack[scrollerID].scrollTimer.mouseAction) {
		scrollerStack[scrollerID].scrollTimer.mouseAction(scrollerID);
	} else if (scrollerStack[scrollerID].scrollTimer.keyAction) {
		scrollerStack[scrollerID].scrollTimer.keyAction(scrollerID);
	} else this.clearTimer(scrollerID);
}


/*
 ************************************************************************
 * debug code uses the div defined in Scroller.html/Scroller.css demo	*
 ************************************************************************
 */
var debugMode = false;

// write to the debug div.
function DEBUG(str) {
	if (debugMode) {
		if (window.widget) {
			alert(str);
		} else {
			var debugDiv = document.getElementById("debugDiv");
			debugDiv.appendChild(document.createTextNode(str));
			debugDiv.appendChild(document.createElement("br"));
			debugDiv.scrollTop = debugDiv.scrollHeight;		
		}
	}
}

// toggle the debugMode flag, but only show the debugDiv if we're in Safari
function toggleDebug() {
	debugMode = !debugMode;
	if (debugMode == true && !window.widget) {
		document.getElementById("debugDiv").style.display = "block";
	} else {
		document.getElementById("debugDiv").style.display = "none";
	}
}
